<?php

namespace App\Http\Controllers\Admin\CMS;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\CMS\Article;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Overtrue\LaravelPinyin\Facades\Pinyin;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\CMS\ArticleRepository;
use App\Repositories\Validators\CMS\ArticleValidator;

/**
 * Class ArticlesController.
 *
 * @package namespace App\Http\Controllers\CMS;
 */
class ArticlesController extends Controller
{
    /**
     * @var ArticleRepository
     */
    protected $repository;

    /**
     * @var ArticleValidator
     */
    protected $validator;

    /**
     * ArticlesController constructor.
     *
     * @param ArticleRepository $repository
     * @param ArticleValidator $validator
     */
    public function __construct(ArticleRepository $repository, ArticleValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    public function index()
    {
        $courses = $this->repository->paginate(request('per_page', self::PAGE_NUM));
        return Response::success($courses);
    }


    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));
            $data['admin_id'] = login_admin_id();
            $data['slug'] = arr2str(Pinyin::permalink($data['title']), '_');
            if (!array_key_exists('published_at', $data) || is_null($data['published_at'])) {
                $data['published_at'] = Carbon::now();
            }

            $information = $this->repository->create($data);
            return Response::success($information);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }


    public function show($id)
    {
        $information = $this->repository->find($id);
        Article::query()->where('id', $information['id'])->increment('view_count');
        return Response::success($information);
    }

    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));
            if (in_array('title', $data)) {
                $data['slug'] = arr2str(Pinyin::permalink($data['title']), '_');
            }

//            if (!in_array('published_at', $data) || empty($data['published_at'])) {
//                $data['published_at'] = Carbon::now();
//            }

            $information = $this->repository->update($data, $id);
            return Response::success($information);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }

    public function destroy($id)
    {
        try {
            $this->repository->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    public function delete(Request $request)
    {
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    public function good(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);

        $information_id = $request->get('id');

        $admin_id = login_admin_id();

        $cache = Cache::get("cache:information:good:{$information_id}:{$admin_id}");
        if ($cache) {
            return Response::fail(T("Do not repeat operation."), ResponseCodeEnum::SYSTEM_ERROR);
        }

        Cache::put("cache:information:good:{$information_id}:{$admin_id}", 1, Carbon::now()->addHours(2));
        return Response::success(null);
    }
}
