<?php

namespace App\Http\Controllers\Admin\TCM;

use App\Http\Controllers\Controller;
use App\Imports\DrugsImport;
use App\Imports\StudentsImport;
use App\Repositories\Criteria\TCM\DrugCriteria;
use App\Repositories\Criteria\TCM\PatientCriteria;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Base\Resource;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Maatwebsite\Excel\Facades\Excel;
use Overtrue\LaravelPinyin\Facades\Pinyin;
use Prettus\Validator\Contracts\ValidatorInterface;
use Prettus\Validator\Exceptions\ValidatorException;
use App\Contracts\Repositories\TCM\DrugRepository;
use App\Repositories\Validators\TCM\DrugValidator;

/**
 * Class DrugsController.
 *
 * @package namespace App\Http\Controllers\TCM;
 */
class DrugController extends Controller
{
    /**
     * @var DrugRepository
     */
    protected $repository;

    /**
     * @var DrugValidator
     */
    protected $validator;

    /**
     * DrugsController constructor.
     *
     * @param DrugRepository $repository
     * @param DrugValidator $validator
     */
    public function __construct(DrugRepository $repository, DrugValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    public function index()
    {
        $lists = $this->repository->paginate(request('per_page', self::PAGE_NUM));
        return Response::success($lists);
    }


    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));
            $data['admin_id'] = login_user_id();
            $letter = Pinyin::abbr($data['name']);
            $data['letter'] = $letter;
            $data['initial'] = substr($letter, 1);
            $drug = $this->repository->create($data);
            return Response::success($drug);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }

    public function show($id)
    {
        $patient = $this->repository->find($id);
        return Response::success($patient);
    }

    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));
            $letter = Pinyin::abbr($data['name']);
            $data['initial'] = substr($letter, 1);
            $drug = $this->repository->update($data, $id);
            return Response::success($drug);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }

    public function destroy($id)
    {
        try {
            $re = $this->repository->delete($id);
            if ($re) {
                return Response::success(null, T('successfully delete.'));
            }
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    public function delete(Request $request)
    {
        try {
            $this->validate($request, [
                'ids' => 'required|array',
            ]);

            $ids = $request->get('ids');
            $this->repository->whereIn('id', $ids)->delete();
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {
            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    public function lists(Request $request)
    {
        $this->repository->pushCriteria(DrugCriteria::class);
        $lists = $this->repository->get();
        return Response::success($lists);
    }

    public function import(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);

        $resource = Resource::query()->where('id', $request->get('id'))->first();
        if (!$resource) {
            return $this->errorFail();
        }

        Excel::import(new DrugsImport(), Storage::disk($resource['disk'])->path($resource['path']));
    }
}
