<?php

namespace App\Http\Controllers\Api\Note;

use App\Http\Controllers\Controller;
use App\Repositories\Enums\ResponseCodeEnum;
use App\Repositories\Models\Inform\Information;
use App\Repositories\Models\Note\Note;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Cache;
use Jiannei\Response\Laravel\Support\Facades\Response;
use Prettus\Validator\Contracts\ValidatorInterface;
use App\Contracts\Repositories\Note\NoteRepository;
use App\Repositories\Validators\Note\NoteValidator;

/**
 * Class NotesController.
 *
 * @package namespace App\Http\Controllers\Note;
 */
class NotesController extends Controller
{
    /**
     * @var NoteRepository
     */
    protected $repository;

    /**
     * @var NoteValidator
     */
    protected $validator;

    /**
     * NotesController constructor.
     *
     * @param NoteRepository $repository
     * @param NoteValidator $validator
     */
    public function __construct(NoteRepository $repository, NoteValidator $validator)
    {
        $this->repository = $repository;
        $this->validator = $validator;
    }

    public function index(Request $request)
    {
        $lists = $this->repository->paginate(request('per_page', self::PAGE_NUM));

        return Response::success($lists);
    }


    public function store(Request $request)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_CREATE));
        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_CREATE)));
            $data['user_id'] = login_user_id();
            $model = $this->repository->create($data);
            return Response::success($model);
        } catch (\Exception $e) {
            return $this->errorStore($e);
        }
    }


    public function show($id)
    {
        $model = $this->repository->find($id);
        Note::query()->where('id', $id)->increment('view_count');
        return Response::success($model);
    }


    public function update(Request $request, $id)
    {
        $this->validate($request, $this->validator->getRules(ValidatorInterface::RULE_UPDATE));

        if (!Note::query()->where('id', $id)->where('user_id', login_user_id())->where('is_release', Note::RELEASED_RAFT)->exists()) {
            return Response::fail('您暂无编辑权限！');
        }

        try {
            $data = $request->only(array_keys($this->validator->getRules(ValidatorInterface::RULE_UPDATE)));
            $model = $this->repository->update($data, $id);
            return Response::success($model);
        } catch (\Exception $e) {
            $this->errorStore($e);
        }
    }

    public function destroy($id)
    {
        if (!Note::query()->where('id', $id)->where('user_id', login_user_id())->exists()) {
            return Response::fail('您暂无删除权限！');
        }
        try {

            $this->repository->delete($id);
            return Response::success(null, T('successfully delete.'));
        } catch (\Exception $exception) {

            return Response::fail(T('Delete failed.'), ResponseCodeEnum::SERVICE_UPDATE_DATA_ERROR);
        }
    }

    public function good(Request $request)
    {
        $this->validate($request, [
            'id' => 'required|integer',
        ]);

        $id = $request->get('id');

        $user_id = login_user_id();

        $cache = Cache::get("cache:note:good:{$id}:{$user_id}");
        if ($cache) {
            return Response::fail(T("Do not repeat operation."), ResponseCodeEnum::SYSTEM_ERROR);
        }

        Note::query()->where('id', $id)->increment('good_count');

        Cache::put("cache:note:good:{$id}:{$user_id}", 1, Carbon::now()->addHours(2));
        return Response::success(null);
    }

}
